# TODO: Add comment
# 
# Author: Paco
###############################################################################


#Solves PRM system of equations with meand dbh and quadratic mean diameter
#Dg length one vector with quadratic mean diameter,
#$Dm, length one vector with mean diameter
#$dmin, length one vector with minimum measured diameter. Default value of 5.
DweibullDm<-function(Dg,Dm,dmin=5){
	
	variance<-Dg^2-Dm^2
	Dmed1<-Dm-dmin
	obj<-function(shape,var){
		scale<-Dmed1/gamma(1+(1/shape))	
		varmod<-(gamma(1+2/shape)-gamma(1+1/shape)^2)*scale^2
		(varmod-var)^2
		
	}
	shape<-try(optimize(obj,c(1.001,500),var=variance))
	
	if(class(shape)=="try-error"){
		shape<-NA
		scale<-NA
		objective<-NA
	}else{
		objective<-shape$objective
		shape<-shape$minimum
		scale<-Dmed1/gamma(1+(1/shape))		
	}
	
	list(shape=shape,scale=scale,objective=objective)
	
}


##Solves PRM system of equations with dominant dbh, quadratic mean diameter and stand density.
#Starting points for the search of solutions are specified in start_shape and start_scale. Provided
#values are used to construct a grid of starting points for the search of solutions
#Dg length one vector with quadratic mean diameter,
#Do, length one vector with dominant diameter
#N, length one vector with stand density in trees per hectare
#dmin, length one vector with minimum measured diameter. Default value of 5.
DweibullDo<-function(Dg,Do,N,dmin=5,
		start_shape=log(c(1.25,7.5,15,20)),start_scale= log(c(2,8,60))){
	
	obj<-function(pars){
		
		shape<-exp(pars[1])
		scale<-exp(pars[2])
		
		Dgest<-try(sqrt(dmin^2+2*scale*dmin*gamma(1+1/shape)
								+gamma(1+2/shape)*(scale^2)))
		lower<-try(qweibull((1-(100/N)),shape,scale))
		Doest<-try({
					qweibull((1-(100/N)),shape,scale)
					a1<-integrate(function(x){(x+5)*dweibull(x,shape,scale)}
							,lower=lower,upper=Inf)
					a1$value/(100/N)
				})
		
		if(inherits(Dgest,"try-error")|inherits(Doest,"try-error")){
			return(10^14)
		}
		if(20*20<(gamma(1+2/shape)-gamma(1+1/shape)^2)*scale^2){
			return(10^14)
		}
		return((Doest-Do)^2+(Dgest-Dg)^2)
		
	}
	
	cont<-1
	issol<-FALSE
	for(i in start_shape){
		for(j in start_scale){
			lastsol<-try(
					optim(c(i,j),obj,method="L-BFGS-B",
							lower=c(log(1.00001),0),
							upper=c(log(500),log(80)),
							control=list(trace=0))
			)
			if(!class(lastsol)=="try-error"){
				if(cont==1){
					issol<-TRUE
					sol<-lastsol
					cont<-cont+1
				}else{
					
					if(lastsol$value<sol$value){
						sol<-lastsol
					}
				}
			}
		}
	}
	if(!issol){
		
		shape<-NA
		scale<-NA
		objective<-NA
		
	}else{
		
		shape<-exp(sol$par[1])
		scale<-exp(sol$par[2])
		objective<-sol$value
	}
	
	list(shape=shape,scale=scale,objective=objective)
	
}


